<?php 
// $Id: views_popup.views.inc,v 1.1.2.1 2008/09/23 16:41:03 bwong Exp $

/**
* Implementation of hook_views_plugin
*
*
*/
function views_popup_views_plugins() {
  return array(
    'style' => array( //declare the views_popup style plugin
      'views_popup' => array(
        'title' => t('Popup'),
        'help' => t('Displays nodes with popup fields'),
        'handler' => 'views_plugin_style_popup',
        'theme' => 'views_view_popup', //'views_view_popup',
        'uses row plugin' => TRUE,
        'uses fields' => TRUE,
        'uses options' => TRUE,
        'type' => 'normal',      
      ),
    )
  );
}



/**
 * Admin settings callback
 */

class views_plugin_style_popup extends views_plugin_style_table {
  function option_definition() {
    $options = parent::option_definition();

    $options['views_popup_exclude'] = array('default' => 1);
    $options['views_popup_delay'] = array('default' => 500);
    $options['views_popup_reset_time'] = array('default' => 500);
    $options['views_popup_follow_mouse'] = array('default' => true);

    return $options;
  }

  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);
  $form['views_popup_exclude'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of fields.'),
    '#default_value' => $this->options['views_popup_exclude'],
    '#field_suffix' => t('fields'),
    '#size' => 12,
    '#description' => t('The number of fields to display initially. The remaining fields will be displayed in a popup.')
  );

  $form['views_popup_delay'] = array(
    '#type' => 'textfield',
    '#title' => t('Popup delay'),
    '#default_value' => $this->options['views_popup_delay'],
    '#field_suffix' => t('milliseconds'),
    '#size' => 12,
    '#description' => t('Delay before popup is shown when mouse hovers over the first field. ')
  );

  $form['views_popup_reset_time'] = array(
    '#type' => 'textfield',
    '#title' => t('Reset timeout'),
    '#default_value' => $this->options['views_popup_reset_time'],
    '#field_suffix' => t('milliseconds'),
    '#size' => 12,
    '#description' => t('Delay after a popup occurs before the popup delay for the next popup is changed from 0 to the value listed above. ')
  );

  $form['views_popup_follow_mouse'] = array(
    '#type' => 'checkbox',
    '#title' => t('Popup follows mouse'),
    '#default_value' => $this->options['views_popup_follow_mouse'],
    '#description' => t('The y position of the tooltip will follow the mouse. If turned off the tooltip will be displayed inline with the appropriate row.')
  );
  }
}


function template_preprocess_views_view_popup(&$vars) {
  $view     = $vars['view'];

  // We need the raw data for this grouping, which is passed in as $vars['rows'].
  // However, the template also needs to use for the rendered fields.  We
  // therefore swap the raw data out to a new variable and reset $vars['rows']
  // so that it can get rebuilt.
  $result   = $vars['rows'];
  $vars['rows'] = array();

  $options  = $view->style_plugin->options;
  $handler  = $view->style_plugin;

  $fields   = &$view->field;
  $columns  = $handler->sanitize_columns($options['columns'], $fields);

  $active   = !empty($handler->active) ? $handler->active : '';
  $order    = !empty($handler->order) ? $handler->order : 'asc';

  $query    = tablesort_get_querystring();
  if ($query) {
    $query = '&' . $query;
  }

  foreach ($columns as $field => $column) {
    // render the header labels
    if ($field == $column && empty($fields[$field]->options['exclude'])) {
      $label = check_plain(!empty($fields[$field]) ? $fields[$field]->label() : '');
      if (empty($options['info'][$field]['sortable'])) {
        $vars['header'][$field] = $label;
      }
      else {
        // @todo -- make this a setting
        $initial = 'asc';

        if ($active == $field && $order == 'asc') {
          $initial = 'desc';
        }

        $image = theme('tablesort_indicator', $initial);
        $title = t('sort by @s', array('@s' => $label));
        $link_options = array(
          'html' => true,
          'attributes' => array('title' => $title),
          'query' => 'order=' . urlencode($field) . '&sort=' . $initial . $query,
        );
        $vars['header'][$field] = l($label . $image, $_GET['q'], $link_options);
      }
    }

    // Create a second variable so we can easily find what fields we have and what the
    // CSS classes should be.
    $vars['fields'][$field] = views_css_safe($field);
    if ($active == $field) {
      $vars['fields'][$field] .= ' active';
    }

    // Render each field into its appropriate column.
    foreach ($result as $num => $row) {
      if (!empty($fields[$field]) && empty($fields[$field]->options['exclude'])) {
        $field_output = $fields[$field]->theme($row);

        // Don't bother with separators and stuff if the field does not show up.
        if (!isset($field_output) && isset($vars['rows'][$num][$column])) {
          continue;
        }

        // Place the field into the column, along with an optional separator.
        if (isset($vars['rows'][$num][$column])) {
          if (!empty($options['info'][$column]['separator'])) {
            $vars['rows'][$num][$column] .= filter_xss_admin($options['info'][$column]['separator']);
          }
        }
        else {
          $vars['rows'][$num][$column] = '';
        }

        $vars['rows'][$num][$column] .= $field_output;
      }
    }
  }

  $vars['class'] = 'views-table';
  if (!empty($options['sticky'])) {
    drupal_add_js('misc/tableheader.js');
    $vars['class'] .= " sticky-enabled";
  }

  /*********************
   * VIEWS POPUP OUTPUT *
  **********************/

  static $id_num = 0; // This is used to set the id of the html elements to popup. We don't want to start from 0 each time or there will be id conflicts on the page when displaying more than one popup view. The second view will start numbering its elements from where the first view left off.

  $results = array();
  $num_displayed = $options['views_popup_exclude']; // The number of fields to display initially (not in popup)
  // this nested loop splits each result object in two (one object to be displayed in the table and the other in the popup)
  foreach($vars['rows'] as $count => $row) {
	  $i = 0;
	  foreach($row as $field => $content) {
		  if($i < $num_displayed) {
			  $row_new[$field] = $content; // The fields to be displayed in a table
		  } else {
			  $popup_new[$field] = $content; // The fields to be displayed in the popup
		  }
		  $i++;
	  }
	  $curr_result['id_num'] = $id_num;
	  $curr_result['content']['displayed'] = $row_new;
	  $curr_result['content']['popup'] = $popup_new;
	  $results[] = $curr_result;
	  $id_num += 1;
  }
  $popup_headers = array(); // Split the table headers in to two arrays, one to be displayed in the table and the second in the popup.
  $popup_headers['displayed'] = array_slice($vars['header'], 0, $num_displayed);
  $popup_headers['popup'] = array_slice($vars['header'], $num_displayed);

  $vars['popup_results'] = $results; // Set the variables to be retrieved in the template.
  $vars['popup_headers'] = $popup_headers;

  drupal_add_css(drupal_get_path('module', 'views_popup') .'/views_popup.css');
  drupal_add_js(array('views_popup' => array( // pass the module options to the javascript that handles the popups
	  'popup_delay' => $options['views_popup_delay'],
	  'reset_time' => $options['views_popup_reset_time'],
	  'follow_mouse' => $options['views_popup_follow_mouse']
  )), 'setting');
  drupal_add_js(drupal_get_path('module', 'views_popup').'/views_popup.js');

}
